// Copyright 2015-2016 Espressif Systems (Shanghai) PTE LTD
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at

//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
#include "esp_types.h"
#include "soc/i2c_reg.h"


#ifndef _DRIVER_I2C_H_
#define _DRIVER_I2C_H_



#define I2C(i)   ((struct I2C_DEV *)(REG_I2C_BASE(i)))


#define I2C_FREQ    100000   //HZ
#define I2C_SLAVE_DEV_ADDR 0x60
#define I2C_APB_CLK_FREQ  80000000



#define I2C_ASSERT(func,warning)   do{ \
	                                 if( (func) == 0 ) {\
									 	ets_printf("PARAM ERROR in %s@%d\r\n",__func__,__LINE__);\
	                                 	}\
                                     }while(0)
#define I2C_SDA_SIG_OUT_NUM(i)    (((i) == 0?)I2CEXT0_SDA_OUT_IDX: \
	                             (((i)==1)?I2CEXT1_SDA_OUT_IDX:I2CEXT0_SDA_OUT_IDX) )
#define I2C_SDA_SIG_IN_NUM(i)    (((i) == 0?)I2CEXT0_SDA_IN_IDX: \
									 (((i)==1)?I2CEXT1_SDA_IN_IDX:I2CEXT0_SDA_IN_IDX) )
#define I2C_GPIO_FUNC   2
#define I2C_SCL_SIG_OUT_NUM(i)    (((i) == 0?)I2CEXT0_SCL_OUT_IDX: \
	                             (((i)==1)?I2CEXT1_SCL_OUT_IDX:I2CEXT0_SCL_OUT_IDX) )
#define I2C_SCL_SIG_IN_NUM(i)    (((i) == 0?)I2CEXT0_SCL_IN_IDX: \
									 (((i)==1)?I2CEXT1_SCL_IN_IDX:I2CEXT0_SCL_IN_IDX) )


typedef struct{
	uint8_t mode;
	uint8_t addr_10bit_en;
	uint16_t slave_addr;
	uint32_t clk_speed;
}I2C_InitTypeDef;

enum CMD_OP_CODE{
	I2C_CMD_RESTART = 0,
	I2C_CMD_WRITE,
	I2C_CMD_READ,
	I2C_CMD_STOP,
	I2C_CMD_END
};

struct I2C_CmdDef{
	uint8_t byte_num;
	uint8_t ack_en;
	uint8_t ack_exp;
	uint8_t ack_val;
	enum CMD_OP_CODE op_code;
};

enum{
	I2C_SLAVE_MODE = 0,
	I2C_MASTER_MODE,

};

#define I2C_COMD_REG(i2c_no, cmd_no) (REG_I2C_BASE(i2c_no) + 0x0058 + (cmd_no)*4 )
    #define I2C_COMMAND_DONE (BIT(31))
    #define I2C_COMMAND_DONE_M (BIT(31))
    #define I2C_COMMAND_DONE_V 0x1
    #define I2C_COMMAND_DONE_S 31
	#define I2C_COMMAND_OP_CODE   0x7
	#define I2C_COMMAND_OP_CODE_M  ((I2C_COMMAND_OP_CODE_V)<<(I2C_COMMAND_OP_CODE_S))
	#define I2C_COMMAND_OP_CODE_V 0x7
	#define I2C_COMMAND_OP_CODE_S  11
	#define I2C_COMMAND_ACK_VALUE   (BIT(10))
	
#define I2C_COMMAND_ACK_VALUE_M (BIT(10))
	#define I2C_COMMAND_ACK_VALUE_V     0x1
	#define I2C_COMMAND_ACK_VALUE_S     10
	
#define I2C_COMMAND_ACK_EXP     (BIT(9))
	#define I2C_COMMAND_ACK_EXP_M   (BIT(9))
	#define I2C_COMMAND_ACK_EXP_V       0x1
	#define I2C_COMMAND_ACK_EXP_S       9
	#define I2C_COMMAND_ACK_EN      (BIT(8))
    #define I2C_COMMAND_ACK_EN_M	(BIT(8))
    #define I2C_COMMAND_ACK_EN_V		0x1
    #define I2C_COMMAND_ACK_EN_S		8
	#define I2C_COMMAND_BYTE_NUM     0xff
    #define I2C_COMMAND_BYTE_NUM_M	 0xff
    #define I2C_COMMAND_BYTE_NUM_V	 0xff
    #define I2C_COMMAND_BYTE_NUM_S	    0	
        #define I2C_GET_COMMAND_DONE(i2c_no,i)     GET_PERI_REG_BITS2(I2C_COMD_REG(i2c_no,i),I2C_COMMAND0_DONE_V,I2C_COMMAND0_DONE_S)
        #define I2C_SET_COMMAND_DONE(i2c_no,i,val) SET_PERI_REG_BITS(I2C_COMD_REG(i2c_no,i),I2C_COMMAND0_DONE_V,val,I2C_COMMAND0_DONE_S)
		#define I2C_GET_COMMAND_OP_CODE(i2c_no,i) GET_PERI_REG_BITS2(I2C_COMD_REG(i2c_no,i),I2C_COMMAND_OP_CODE_V,I2C_COMMAND_OP_CODE_S)
        #define I2C_SET_COMMAND_OP_CODE(i2c_no,i,val) SET_PERI_REG_BITS(I2C_COMD_REG(i2c_no,i),I2C_COMMAND_OP_CODE_V,val,I2C_COMMAND_OP_CODE_S)
        #define I2C_GET_COMMAND_ACK_VALUE(i2c_no,i) GET_PERI_REG_BITS2(I2C_COMD_REG(i2c_no,i),I2C_COMMAND_ACK_VALUE_V,I2C_COMMAND_ACK_VALUE_S)
        #define I2C_SET_COMMAND_ACK_VALUE(i2c_no,i,val) SET_PERI_REG_BITS(I2C_COMD_REG(i2c_no,i),I2C_COMMAND_ACK_VALUE_V,val,I2C_COMMAND_ACK_VALUE_S)
        #define I2C_GET_COMMAND_ACK_EXP(i2c_no,i) GET_PERI_REG_BITS2(I2C_COMD_REG(i2c_no,i),I2C_COMMAND_ACK_EXP_V,I2C_COMMAND_ACK_EXP_S)
        #define I2C_SET_COMMAND_ACK_EXP(i2c_no,i,val) SET_PERI_REG_BITS(I2C_COMD_REG(i2c_no,i),I2C_COMMAND_ACK_EXP_V,val,I2C_COMMAND_ACK_EXP_S)
        #define I2C_GET_COMMAND_ACK_EN(i2c_no,i) GET_PERI_REG_BITS2(I2C_COMD_REG(i2c_no,i),I2C_COMMAND_ACK_EN_V,I2C_COMMAND_ACK_EN_S)
        #define I2C_SET_COMMAND_ACK_EN(i2c_no,i,val) SET_PERI_REG_BITS(I2C_COMD_REG(i2c_no,i),I2C_COMMAND_ACK_EN_V,val,I2C_COMMAND_ACK_EN_S)
        #define I2C_GET_COMMAND_BYTE_NUM(i2c_no,i) GET_PERI_REG_BITS2(I2C_COMD_REG(i2c_no,i),I2C_COMMAND_BYTE_NUM_V,I2C_COMMAND_BYTE_NUM_S)
        #define I2C_SET_COMMAND_BYTE_NUM(i2c_no,i,val) SET_PERI_REG_BITS(I2C_COMD_REG(i2c_no,i),I2C_COMMAND_BYTE_NUM_V,val,I2C_COMMAND_BYTE_NUM_S)


		
#define I2C_GET_SCL_LOW_PERIOD(i)  GET_PERI_REG_BITS2(I2C_SCL_LOW_PERIOD_REG(i) , I2C_SCL_LOW_PERIOD_V, I2C_SCL_LOW_PERIOD_S)
#define I2C_SET_SCL_LOW_PERIOD(i,val)  SET_PERI_REG_BITS(I2C_SCL_LOW_PERIOD_REG(i) ,I2C_SCL_LOW_PERIOD_V,(val),I2C_SCL_LOW_PERIOD_S)
		
#define I2C_GET_CLK_EN(i)  GET_PERI_REG_BITS2(I2C_CTR_REG(i) , I2C_CLK_EN_V, I2C_CLK_EN_S)
#define I2C_SET_CLK_EN(i,val)  SET_PERI_REG_BITS(I2C_CTR_REG(i) ,I2C_CLK_EN_V,(val),I2C_CLK_EN_S)
#define I2C_GET_RX_LSB_FIRST(i)  GET_PERI_REG_BITS2(I2C_CTR_REG(i) , I2C_RX_LSB_FIRST_V, I2C_RX_LSB_FIRST_S)
#define I2C_SET_RX_LSB_FIRST(i,val)  SET_PERI_REG_BITS(I2C_CTR_REG(i) ,I2C_RX_LSB_FIRST_V,(val),I2C_RX_LSB_FIRST_S)
#define I2C_GET_TX_LSB_FIRST(i)  GET_PERI_REG_BITS2(I2C_CTR_REG(i) , I2C_TX_LSB_FIRST_V, I2C_TX_LSB_FIRST_S)
#define I2C_SET_TX_LSB_FIRST(i,val)  SET_PERI_REG_BITS(I2C_CTR_REG(i) ,I2C_TX_LSB_FIRST_V,(val),I2C_TX_LSB_FIRST_S)
#define I2C_GET_TRANS_START(i)  GET_PERI_REG_BITS2(I2C_CTR_REG(i) , I2C_TRANS_START_V, I2C_TRANS_START_S)
#define I2C_SET_TRANS_START(i,val)  SET_PERI_REG_BITS(I2C_CTR_REG(i) ,I2C_TRANS_START_V,(val),I2C_TRANS_START_S)
#define I2C_GET_MS_MODE(i)  GET_PERI_REG_BITS2(I2C_CTR_REG(i) , I2C_MS_MODE_V, I2C_MS_MODE_S)
#define I2C_SET_MS_MODE(i,val)  SET_PERI_REG_BITS(I2C_CTR_REG(i) ,I2C_MS_MODE_V,(val),I2C_MS_MODE_S)
#define I2C_GET_SAMPLE_SCL_LEVEL(i)  GET_PERI_REG_BITS2(I2C_CTR_REG(i) , I2C_SAMPLE_SCL_LEVEL_V, I2C_SAMPLE_SCL_LEVEL_S)
#define I2C_SET_SAMPLE_SCL_LEVEL(i,val)  SET_PERI_REG_BITS(I2C_CTR_REG(i) ,I2C_SAMPLE_SCL_LEVEL_V,(val),I2C_SAMPLE_SCL_LEVEL_S)
#define I2C_GET_SCL_FORCE_OUT(i)  GET_PERI_REG_BITS2(I2C_CTR_REG(i) , I2C_SCL_FORCE_OUT_V, I2C_SCL_FORCE_OUT_S)
#define I2C_SET_SCL_FORCE_OUT(i,val)  SET_PERI_REG_BITS(I2C_CTR_REG(i) ,I2C_SCL_FORCE_OUT_V,(val),I2C_SCL_FORCE_OUT_S)
#define I2C_GET_SDA_FORCE_OUT(i)  GET_PERI_REG_BITS2(I2C_CTR_REG(i) , I2C_SDA_FORCE_OUT_V, I2C_SDA_FORCE_OUT_S)
#define I2C_SET_SDA_FORCE_OUT(i,val)  SET_PERI_REG_BITS(I2C_CTR_REG(i) ,I2C_SDA_FORCE_OUT_V,(val),I2C_SDA_FORCE_OUT_S)
		
#define I2C_GET_SCL_STATE_LAST(i)  GET_PERI_REG_BITS2(I2C_SR_REG(i) , I2C_SCL_STATE_LAST_V, I2C_SCL_STATE_LAST_S)
#define I2C_GET_SCL_MAIN_STATE_LAST(i)  GET_PERI_REG_BITS2(I2C_SR_REG(i) , I2C_SCL_MAIN_STATE_LAST_V, I2C_SCL_MAIN_STATE_LAST_S)
#define I2C_GET_TXFIFO_CNT(i)  GET_PERI_REG_BITS2(I2C_SR_REG(i) , I2C_TXFIFO_CNT_V, I2C_TXFIFO_CNT_S)
#define I2C_GET_RXFIFO_CNT(i)  GET_PERI_REG_BITS2(I2C_SR_REG(i) , I2C_RXFIFO_CNT_V, I2C_RXFIFO_CNT_S)
#define I2C_GET_BYTE_TRANS(i)  GET_PERI_REG_BITS2(I2C_SR_REG(i) , I2C_BYTE_TRANS_V, I2C_BYTE_TRANS_S)
#define I2C_GET_SLAVE_ADDRESSED(i)  GET_PERI_REG_BITS2(I2C_SR_REG(i) , I2C_SLAVE_ADDRESSED_V, I2C_SLAVE_ADDRESSED_S)
#define I2C_GET_BUS_BUSY(i)  GET_PERI_REG_BITS2(I2C_SR_REG(i) , I2C_BUS_BUSY_V, I2C_BUS_BUSY_S)
#define I2C_GET_ARB_LOST(i)  GET_PERI_REG_BITS2(I2C_SR_REG(i) , I2C_ARB_LOST_V, I2C_ARB_LOST_S)
#define I2C_GET_TIME_OUT(i)  GET_PERI_REG_BITS2(I2C_SR_REG(i) , I2C_TIME_OUT_V, I2C_TIME_OUT_S)
#define I2C_GET_SLAVE_RW(i)  GET_PERI_REG_BITS2(I2C_SR_REG(i) , I2C_SLAVE_RW_V, I2C_SLAVE_RW_S)
#define I2C_GET_ACK_REC(i)  GET_PERI_REG_BITS2(I2C_SR_REG(i) , I2C_ACK_REC_V, I2C_ACK_REC_S)
		
#define I2C_GET_TIME_OUT_REG(i)  GET_PERI_REG_BITS2(I2C_TO_REG(i) , I2C_TIME_OUT_REG_V, I2C_TIME_OUT_REG_S)
#define I2C_SET_TIME_OUT_REG(i,val)  SET_PERI_REG_BITS(I2C_TO_REG(i) ,I2C_TIME_OUT_REG_V,(val),I2C_TIME_OUT_REG_S)
		
#define I2C_GET_ADDR_10BIT_EN(i)  GET_PERI_REG_BITS2(I2C_SLAVE_ADDR_REG(i) , I2C_ADDR_10BIT_EN_V, I2C_ADDR_10BIT_EN_S)
#define I2C_SET_ADDR_10BIT_EN(i,val)  SET_PERI_REG_BITS(I2C_SLAVE_ADDR_REG(i) ,I2C_ADDR_10BIT_EN_V,(val),I2C_ADDR_10BIT_EN_S)
#define I2C_GET_SLAVE_ADDR(i)  GET_PERI_REG_BITS2(I2C_SLAVE_ADDR_REG(i) , I2C_SLAVE_ADDR_V, I2C_SLAVE_ADDR_S)
#define I2C_SET_SLAVE_ADDR(i,val)  SET_PERI_REG_BITS(I2C_SLAVE_ADDR_REG(i) ,I2C_SLAVE_ADDR_V,(val),I2C_SLAVE_ADDR_S)
		
#define I2C_GET_TXFIFO_END_ADDR(i)  GET_PERI_REG_BITS2(I2C_RXFIFO_ST_REG(i) , I2C_TXFIFO_END_ADDR_V, I2C_TXFIFO_END_ADDR_S)
#define I2C_GET_TXFIFO_START_ADDR(i)  GET_PERI_REG_BITS2(I2C_RXFIFO_ST_REG(i) , I2C_TXFIFO_START_ADDR_V, I2C_TXFIFO_START_ADDR_S)
#define I2C_GET_RXFIFO_END_ADDR(i)  GET_PERI_REG_BITS2(I2C_RXFIFO_ST_REG(i) , I2C_RXFIFO_END_ADDR_V, I2C_RXFIFO_END_ADDR_S)
#define I2C_GET_RXFIFO_START_ADDR(i)  GET_PERI_REG_BITS2(I2C_RXFIFO_ST_REG(i) , I2C_RXFIFO_START_ADDR_V, I2C_RXFIFO_START_ADDR_S)
		
#define I2C_GET_NONFIFO_TX_THRES(i)  GET_PERI_REG_BITS2(I2C_FIFO_CONF_REG(i) , I2C_NONFIFO_TX_THRES_V, I2C_NONFIFO_TX_THRES_S)
#define I2C_SET_NONFIFO_TX_THRES(i,val)  SET_PERI_REG_BITS(I2C_FIFO_CONF_REG(i) ,I2C_NONFIFO_TX_THRES_V,(val),I2C_NONFIFO_TX_THRES_S)
#define I2C_GET_NONFIFO_RX_THRES(i)  GET_PERI_REG_BITS2(I2C_FIFO_CONF_REG(i) , I2C_NONFIFO_RX_THRES_V, I2C_NONFIFO_RX_THRES_S)
#define I2C_SET_NONFIFO_RX_THRES(i,val)  SET_PERI_REG_BITS(I2C_FIFO_CONF_REG(i) ,I2C_NONFIFO_RX_THRES_V,(val),I2C_NONFIFO_RX_THRES_S)
#define I2C_GET_TX_FIFO_RST(i)  GET_PERI_REG_BITS2(I2C_FIFO_CONF_REG(i) , I2C_TX_FIFO_RST_V, I2C_TX_FIFO_RST_S)
#define I2C_SET_TX_FIFO_RST(i,val)  SET_PERI_REG_BITS(I2C_FIFO_CONF_REG(i) ,I2C_TX_FIFO_RST_V,(val),I2C_TX_FIFO_RST_S)
#define I2C_GET_RX_FIFO_RST(i)  GET_PERI_REG_BITS2(I2C_FIFO_CONF_REG(i) , I2C_RX_FIFO_RST_V, I2C_RX_FIFO_RST_S)
#define I2C_SET_RX_FIFO_RST(i,val)  SET_PERI_REG_BITS(I2C_FIFO_CONF_REG(i) ,I2C_RX_FIFO_RST_V,(val),I2C_RX_FIFO_RST_S)
#define I2C_GET_FIFO_ADDR_CFG_EN(i)  GET_PERI_REG_BITS2(I2C_FIFO_CONF_REG(i) , I2C_FIFO_ADDR_CFG_EN_V, I2C_FIFO_ADDR_CFG_EN_S)
#define I2C_SET_FIFO_ADDR_CFG_EN(i,val)  SET_PERI_REG_BITS(I2C_FIFO_CONF_REG(i) ,I2C_FIFO_ADDR_CFG_EN_V,(val),I2C_FIFO_ADDR_CFG_EN_S)
#define I2C_GET_NONFIFO_EN(i)  GET_PERI_REG_BITS2(I2C_FIFO_CONF_REG(i) , I2C_NONFIFO_EN_V, I2C_NONFIFO_EN_S)
#define I2C_SET_NONFIFO_EN(i,val)  SET_PERI_REG_BITS(I2C_FIFO_CONF_REG(i) ,I2C_NONFIFO_EN_V,(val),I2C_NONFIFO_EN_S)
#define I2C_GET_TXFIFO_EMPTY_THRHD(i)  GET_PERI_REG_BITS2(I2C_FIFO_CONF_REG(i) , I2C_TXFIFO_EMPTY_THRHD_V, I2C_TXFIFO_EMPTY_THRHD_S)
#define I2C_SET_TXFIFO_EMPTY_THRHD(i,val)  SET_PERI_REG_BITS(I2C_FIFO_CONF_REG(i) ,I2C_TXFIFO_EMPTY_THRHD_V,(val),I2C_TXFIFO_EMPTY_THRHD_S)
#define I2C_GET_RXFIFO_FULL_THRHD(i)  GET_PERI_REG_BITS2(I2C_FIFO_CONF_REG(i) , I2C_RXFIFO_FULL_THRHD_V, I2C_RXFIFO_FULL_THRHD_S)
#define I2C_SET_RXFIFO_FULL_THRHD(i,val)  SET_PERI_REG_BITS(I2C_FIFO_CONF_REG(i) ,I2C_RXFIFO_FULL_THRHD_V,(val),I2C_RXFIFO_FULL_THRHD_S)
		
#define I2C_GET_FIFO_RDATA(i)  GET_PERI_REG_BITS2(I2C_DATA_REG(i) , I2C_FIFO_RDATA_V, I2C_FIFO_RDATA_S)
		
#define I2C_GET_TX_SEND_EMPTY_INT_RAW(i)  GET_PERI_REG_BITS2(I2C_INT_RAW_REG(i) , I2C_TX_SEND_EMPTY_INT_RAW_V, I2C_TX_SEND_EMPTY_INT_RAW_S)
#define I2C_GET_RX_REC_FULL_INT_RAW(i)  GET_PERI_REG_BITS2(I2C_INT_RAW_REG(i) , I2C_RX_REC_FULL_INT_RAW_V, I2C_RX_REC_FULL_INT_RAW_S)
#define I2C_GET_ACK_ERR_INT_RAW(i)  GET_PERI_REG_BITS2(I2C_INT_RAW_REG(i) , I2C_ACK_ERR_INT_RAW_V, I2C_ACK_ERR_INT_RAW_S)
#define I2C_GET_TRANS_START_INT_RAW(i)  GET_PERI_REG_BITS2(I2C_INT_RAW_REG(i) , I2C_TRANS_START_INT_RAW_V, I2C_TRANS_START_INT_RAW_S)
#define I2C_GET_TIME_OUT_INT_RAW(i)  GET_PERI_REG_BITS2(I2C_INT_RAW_REG(i) , I2C_TIME_OUT_INT_RAW_V, I2C_TIME_OUT_INT_RAW_S)
#define I2C_GET_TRANS_COMPLETE_INT_RAW(i)  GET_PERI_REG_BITS2(I2C_INT_RAW_REG(i) , I2C_TRANS_COMPLETE_INT_RAW_V, I2C_TRANS_COMPLETE_INT_RAW_S)
#define I2C_GET_MASTER_TRAN_COMP_INT_RAW(i)  GET_PERI_REG_BITS2(I2C_INT_RAW_REG(i) , I2C_MASTER_TRAN_COMP_INT_RAW_V, I2C_MASTER_TRAN_COMP_INT_RAW_S)
#define I2C_GET_ARBITRATION_LOST_INT_RAW(i)  GET_PERI_REG_BITS2(I2C_INT_RAW_REG(i) , I2C_ARBITRATION_LOST_INT_RAW_V, I2C_ARBITRATION_LOST_INT_RAW_S)
#define I2C_GET_SLAVE_TRAN_COMP_INT_RAW(i)  GET_PERI_REG_BITS2(I2C_INT_RAW_REG(i) , I2C_SLAVE_TRAN_COMP_INT_RAW_V, I2C_SLAVE_TRAN_COMP_INT_RAW_S)
#define I2C_GET_END_DETECT_INT_RAW(i)  GET_PERI_REG_BITS2(I2C_INT_RAW_REG(i) , I2C_END_DETECT_INT_RAW_V, I2C_END_DETECT_INT_RAW_S)
#define I2C_GET_RXFIFO_OVF_INT_RAW(i)  GET_PERI_REG_BITS2(I2C_INT_RAW_REG(i) , I2C_RXFIFO_OVF_INT_RAW_V, I2C_RXFIFO_OVF_INT_RAW_S)
#define I2C_GET_TXFIFO_EMPTY_INT_RAW(i)  GET_PERI_REG_BITS2(I2C_INT_RAW_REG(i) , I2C_TXFIFO_EMPTY_INT_RAW_V, I2C_TXFIFO_EMPTY_INT_RAW_S)
#define I2C_GET_RXFIFO_FULL_INT_RAW(i)  GET_PERI_REG_BITS2(I2C_INT_RAW_REG(i) , I2C_RXFIFO_FULL_INT_RAW_V, I2C_RXFIFO_FULL_INT_RAW_S)
		
#define I2C_SET_TX_SEND_EMPTY_INT_CLR(i,val)  WRITE_PERI_REG(I2C_INT_CLR_REG(i),((val) & I2C_TX_SEND_EMPTY_INT_CLR_V )<<(I2C_TX_SEND_EMPTY_INT_CLR_S))
#define I2C_SET_RX_REC_FULL_INT_CLR(i,val)  WRITE_PERI_REG(I2C_INT_CLR_REG(i),((val) & I2C_RX_REC_FULL_INT_CLR_V )<<(I2C_RX_REC_FULL_INT_CLR_S))
#define I2C_SET_ACK_ERR_INT_CLR(i,val)  WRITE_PERI_REG(I2C_INT_CLR_REG(i),((val) & I2C_ACK_ERR_INT_CLR_V )<<(I2C_ACK_ERR_INT_CLR_S))
#define I2C_SET_TRANS_START_INT_CLR(i,val)  WRITE_PERI_REG(I2C_INT_CLR_REG(i),((val) & I2C_TRANS_START_INT_CLR_V )<<(I2C_TRANS_START_INT_CLR_S))
#define I2C_SET_TIME_OUT_INT_CLR(i,val)  WRITE_PERI_REG(I2C_INT_CLR_REG(i),((val) & I2C_TIME_OUT_INT_CLR_V )<<(I2C_TIME_OUT_INT_CLR_S))
#define I2C_SET_TRANS_COMPLETE_INT_CLR(i,val)  WRITE_PERI_REG(I2C_INT_CLR_REG(i),((val) & I2C_TRANS_COMPLETE_INT_CLR_V )<<(I2C_TRANS_COMPLETE_INT_CLR_S))
#define I2C_SET_MASTER_TRAN_COMP_INT_CLR(i,val)  WRITE_PERI_REG(I2C_INT_CLR_REG(i),((val) & I2C_MASTER_TRAN_COMP_INT_CLR_V )<<(I2C_MASTER_TRAN_COMP_INT_CLR_S))
#define I2C_SET_ARBITRATION_LOST_INT_CLR(i,val)  WRITE_PERI_REG(I2C_INT_CLR_REG(i),((val) & I2C_ARBITRATION_LOST_INT_CLR_V )<<(I2C_ARBITRATION_LOST_INT_CLR_S))
#define I2C_SET_SLAVE_TRAN_COMP_INT_CLR(i,val)  WRITE_PERI_REG(I2C_INT_CLR_REG(i),((val) & I2C_SLAVE_TRAN_COMP_INT_CLR_V )<<(I2C_SLAVE_TRAN_COMP_INT_CLR_S))
#define I2C_SET_END_DETECT_INT_CLR(i,val)  WRITE_PERI_REG(I2C_INT_CLR_REG(i),((val) & I2C_END_DETECT_INT_CLR_V )<<(I2C_END_DETECT_INT_CLR_S))
#define I2C_SET_RXFIFO_OVF_INT_CLR(i,val)  WRITE_PERI_REG(I2C_INT_CLR_REG(i),((val) & I2C_RXFIFO_OVF_INT_CLR_V )<<(I2C_RXFIFO_OVF_INT_CLR_S))
#define I2C_SET_TXFIFO_EMPTY_INT_CLR(i,val)  WRITE_PERI_REG(I2C_INT_CLR_REG(i),((val) & I2C_TXFIFO_EMPTY_INT_CLR_V )<<(I2C_TXFIFO_EMPTY_INT_CLR_S))
#define I2C_SET_RXFIFO_FULL_INT_CLR(i,val)  WRITE_PERI_REG(I2C_INT_CLR_REG(i),((val) & I2C_RXFIFO_FULL_INT_CLR_V )<<(I2C_RXFIFO_FULL_INT_CLR_S))
		
#define I2C_GET_TX_SEND_EMPTY_INT_ENA(i)  GET_PERI_REG_BITS2(I2C_INT_ENA_REG(i) , I2C_TX_SEND_EMPTY_INT_ENA_V, I2C_TX_SEND_EMPTY_INT_ENA_S)
#define I2C_SET_TX_SEND_EMPTY_INT_ENA(i,val)  SET_PERI_REG_BITS(I2C_INT_ENA_REG(i) ,I2C_TX_SEND_EMPTY_INT_ENA_V,(val),I2C_TX_SEND_EMPTY_INT_ENA_S)
#define I2C_GET_RX_REC_FULL_INT_ENA(i)  GET_PERI_REG_BITS2(I2C_INT_ENA_REG(i) , I2C_RX_REC_FULL_INT_ENA_V, I2C_RX_REC_FULL_INT_ENA_S)
#define I2C_SET_RX_REC_FULL_INT_ENA(i,val)  SET_PERI_REG_BITS(I2C_INT_ENA_REG(i) ,I2C_RX_REC_FULL_INT_ENA_V,(val),I2C_RX_REC_FULL_INT_ENA_S)
#define I2C_GET_ACK_ERR_INT_ENA(i)  GET_PERI_REG_BITS2(I2C_INT_ENA_REG(i) , I2C_ACK_ERR_INT_ENA_V, I2C_ACK_ERR_INT_ENA_S)
#define I2C_SET_ACK_ERR_INT_ENA(i,val)  SET_PERI_REG_BITS(I2C_INT_ENA_REG(i) ,I2C_ACK_ERR_INT_ENA_V,(val),I2C_ACK_ERR_INT_ENA_S)
#define I2C_GET_TRANS_START_INT_ENA(i)  GET_PERI_REG_BITS2(I2C_INT_ENA_REG(i) , I2C_TRANS_START_INT_ENA_V, I2C_TRANS_START_INT_ENA_S)
#define I2C_SET_TRANS_START_INT_ENA(i,val)  SET_PERI_REG_BITS(I2C_INT_ENA_REG(i) ,I2C_TRANS_START_INT_ENA_V,(val),I2C_TRANS_START_INT_ENA_S)
#define I2C_GET_TIME_OUT_INT_ENA(i)  GET_PERI_REG_BITS2(I2C_INT_ENA_REG(i) , I2C_TIME_OUT_INT_ENA_V, I2C_TIME_OUT_INT_ENA_S)
#define I2C_SET_TIME_OUT_INT_ENA(i,val)  SET_PERI_REG_BITS(I2C_INT_ENA_REG(i) ,I2C_TIME_OUT_INT_ENA_V,(val),I2C_TIME_OUT_INT_ENA_S)
#define I2C_GET_TRANS_COMPLETE_INT_ENA(i)  GET_PERI_REG_BITS2(I2C_INT_ENA_REG(i) , I2C_TRANS_COMPLETE_INT_ENA_V, I2C_TRANS_COMPLETE_INT_ENA_S)
#define I2C_SET_TRANS_COMPLETE_INT_ENA(i,val)  SET_PERI_REG_BITS(I2C_INT_ENA_REG(i) ,I2C_TRANS_COMPLETE_INT_ENA_V,(val),I2C_TRANS_COMPLETE_INT_ENA_S)
#define I2C_GET_MASTER_TRAN_COMP_INT_ENA(i)  GET_PERI_REG_BITS2(I2C_INT_ENA_REG(i) , I2C_MASTER_TRAN_COMP_INT_ENA_V, I2C_MASTER_TRAN_COMP_INT_ENA_S)
#define I2C_SET_MASTER_TRAN_COMP_INT_ENA(i,val)  SET_PERI_REG_BITS(I2C_INT_ENA_REG(i) ,I2C_MASTER_TRAN_COMP_INT_ENA_V,(val),I2C_MASTER_TRAN_COMP_INT_ENA_S)
#define I2C_GET_ARBITRATION_LOST_INT_ENA(i)  GET_PERI_REG_BITS2(I2C_INT_ENA_REG(i) , I2C_ARBITRATION_LOST_INT_ENA_V, I2C_ARBITRATION_LOST_INT_ENA_S)
#define I2C_SET_ARBITRATION_LOST_INT_ENA(i,val)  SET_PERI_REG_BITS(I2C_INT_ENA_REG(i) ,I2C_ARBITRATION_LOST_INT_ENA_V,(val),I2C_ARBITRATION_LOST_INT_ENA_S)
#define I2C_GET_SLAVE_TRAN_COMP_INT_ENA(i)  GET_PERI_REG_BITS2(I2C_INT_ENA_REG(i) , I2C_SLAVE_TRAN_COMP_INT_ENA_V, I2C_SLAVE_TRAN_COMP_INT_ENA_S)
#define I2C_SET_SLAVE_TRAN_COMP_INT_ENA(i,val)  SET_PERI_REG_BITS(I2C_INT_ENA_REG(i) ,I2C_SLAVE_TRAN_COMP_INT_ENA_V,(val),I2C_SLAVE_TRAN_COMP_INT_ENA_S)
#define I2C_GET_END_DETECT_INT_ENA(i)  GET_PERI_REG_BITS2(I2C_INT_ENA_REG(i) , I2C_END_DETECT_INT_ENA_V, I2C_END_DETECT_INT_ENA_S)
#define I2C_SET_END_DETECT_INT_ENA(i,val)  SET_PERI_REG_BITS(I2C_INT_ENA_REG(i) ,I2C_END_DETECT_INT_ENA_V,(val),I2C_END_DETECT_INT_ENA_S)
#define I2C_GET_RXFIFO_OVF_INT_ENA(i)  GET_PERI_REG_BITS2(I2C_INT_ENA_REG(i) , I2C_RXFIFO_OVF_INT_ENA_V, I2C_RXFIFO_OVF_INT_ENA_S)
#define I2C_SET_RXFIFO_OVF_INT_ENA(i,val)  SET_PERI_REG_BITS(I2C_INT_ENA_REG(i) ,I2C_RXFIFO_OVF_INT_ENA_V,(val),I2C_RXFIFO_OVF_INT_ENA_S)
#define I2C_GET_TXFIFO_EMPTY_INT_ENA(i)  GET_PERI_REG_BITS2(I2C_INT_ENA_REG(i) , I2C_TXFIFO_EMPTY_INT_ENA_V, I2C_TXFIFO_EMPTY_INT_ENA_S)
#define I2C_SET_TXFIFO_EMPTY_INT_ENA(i,val)  SET_PERI_REG_BITS(I2C_INT_ENA_REG(i) ,I2C_TXFIFO_EMPTY_INT_ENA_V,(val),I2C_TXFIFO_EMPTY_INT_ENA_S)
#define I2C_GET_RXFIFO_FULL_INT_ENA(i)  GET_PERI_REG_BITS2(I2C_INT_ENA_REG(i) , I2C_RXFIFO_FULL_INT_ENA_V, I2C_RXFIFO_FULL_INT_ENA_S)
#define I2C_SET_RXFIFO_FULL_INT_ENA(i,val)  SET_PERI_REG_BITS(I2C_INT_ENA_REG(i) ,I2C_RXFIFO_FULL_INT_ENA_V,(val),I2C_RXFIFO_FULL_INT_ENA_S)
		
#define I2C_GET_TX_SEND_EMPTY_INT_ST(i)  GET_PERI_REG_BITS2(I2C_INT_STATUS_REG(i) , I2C_TX_SEND_EMPTY_INT_ST_V, I2C_TX_SEND_EMPTY_INT_ST_S)
#define I2C_GET_RX_REC_FULL_INT_ST(i)  GET_PERI_REG_BITS2(I2C_INT_STATUS_REG(i) , I2C_RX_REC_FULL_INT_ST_V, I2C_RX_REC_FULL_INT_ST_S)
#define I2C_GET_ACK_ERR_INT_ST(i)  GET_PERI_REG_BITS2(I2C_INT_STATUS_REG(i) , I2C_ACK_ERR_INT_ST_V, I2C_ACK_ERR_INT_ST_S)
#define I2C_GET_TRANS_START_INT_ST(i)  GET_PERI_REG_BITS2(I2C_INT_STATUS_REG(i) , I2C_TRANS_START_INT_ST_V, I2C_TRANS_START_INT_ST_S)
#define I2C_GET_TIME_OUT_INT_ST(i)  GET_PERI_REG_BITS2(I2C_INT_STATUS_REG(i) , I2C_TIME_OUT_INT_ST_V, I2C_TIME_OUT_INT_ST_S)
#define I2C_GET_TRANS_COMPLETE_INT_ST(i)  GET_PERI_REG_BITS2(I2C_INT_STATUS_REG(i) , I2C_TRANS_COMPLETE_INT_ST_V, I2C_TRANS_COMPLETE_INT_ST_S)
#define I2C_GET_MASTER_TRAN_COMP_INT_ST(i)  GET_PERI_REG_BITS2(I2C_INT_STATUS_REG(i) , I2C_MASTER_TRAN_COMP_INT_ST_V, I2C_MASTER_TRAN_COMP_INT_ST_S)
#define I2C_GET_ARBITRATION_LOST_INT_ST(i)  GET_PERI_REG_BITS2(I2C_INT_STATUS_REG(i) , I2C_ARBITRATION_LOST_INT_ST_V, I2C_ARBITRATION_LOST_INT_ST_S)
#define I2C_GET_SLAVE_TRAN_COMP_INT_ST(i)  GET_PERI_REG_BITS2(I2C_INT_STATUS_REG(i) , I2C_SLAVE_TRAN_COMP_INT_ST_V, I2C_SLAVE_TRAN_COMP_INT_ST_S)
#define I2C_GET_END_DETECT_INT_ST(i)  GET_PERI_REG_BITS2(I2C_INT_STATUS_REG(i) , I2C_END_DETECT_INT_ST_V, I2C_END_DETECT_INT_ST_S)
#define I2C_GET_RXFIFO_OVF_INT_ST(i)  GET_PERI_REG_BITS2(I2C_INT_STATUS_REG(i) , I2C_RXFIFO_OVF_INT_ST_V, I2C_RXFIFO_OVF_INT_ST_S)
#define I2C_GET_TXFIFO_EMPTY_INT_ST(i)  GET_PERI_REG_BITS2(I2C_INT_STATUS_REG(i) , I2C_TXFIFO_EMPTY_INT_ST_V, I2C_TXFIFO_EMPTY_INT_ST_S)
#define I2C_GET_RXFIFO_FULL_INT_ST(i)  GET_PERI_REG_BITS2(I2C_INT_STATUS_REG(i) , I2C_RXFIFO_FULL_INT_ST_V, I2C_RXFIFO_FULL_INT_ST_S)
		
#define I2C_GET_SDA_HOLD_TIME(i)  GET_PERI_REG_BITS2(I2C_SDA_HOLD_REG(i) , I2C_SDA_HOLD_TIME_V, I2C_SDA_HOLD_TIME_S)
#define I2C_SET_SDA_HOLD_TIME(i,val)  SET_PERI_REG_BITS(I2C_SDA_HOLD_REG(i) ,I2C_SDA_HOLD_TIME_V,(val),I2C_SDA_HOLD_TIME_S)
		
#define I2C_GET_SDA_SAMPLE_TIME(i)  GET_PERI_REG_BITS2(I2C_SDA_SAMPLE_REG(i) , I2C_SDA_SAMPLE_TIME_V, I2C_SDA_SAMPLE_TIME_S)
#define I2C_SET_SDA_SAMPLE_TIME(i,val)  SET_PERI_REG_BITS(I2C_SDA_SAMPLE_REG(i) ,I2C_SDA_SAMPLE_TIME_V,(val),I2C_SDA_SAMPLE_TIME_S)
		
#define I2C_GET_SCL_HIGH_PERIOD(i)  GET_PERI_REG_BITS2(I2C_SCL_HIGH_PERIOD_REG(i) , I2C_SCL_HIGH_PERIOD_V, I2C_SCL_HIGH_PERIOD_S)
#define I2C_SET_SCL_HIGH_PERIOD(i,val)  SET_PERI_REG_BITS(I2C_SCL_HIGH_PERIOD_REG(i) ,I2C_SCL_HIGH_PERIOD_V,(val),I2C_SCL_HIGH_PERIOD_S)
		
#define I2C_GET_SCL_START_HOLD_TIME(i)  GET_PERI_REG_BITS2(I2C_SCL_START_HOLD_REG(i) , I2C_SCL_START_HOLD_TIME_V, I2C_SCL_START_HOLD_TIME_S)
#define I2C_SET_SCL_START_HOLD_TIME(i,val)  SET_PERI_REG_BITS(I2C_SCL_START_HOLD_REG(i) ,I2C_SCL_START_HOLD_TIME_V,(val),I2C_SCL_START_HOLD_TIME_S)
		
#define I2C_GET_SCL_RSTART_SETUP_TIME(i)  GET_PERI_REG_BITS2(I2C_SCL_RSTART_SETUP_REG(i) , I2C_SCL_RSTART_SETUP_TIME_V, I2C_SCL_RSTART_SETUP_TIME_S)
#define I2C_SET_SCL_RSTART_SETUP_TIME(i,val)  SET_PERI_REG_BITS(I2C_SCL_RSTART_SETUP_REG(i) ,I2C_SCL_RSTART_SETUP_TIME_V,(val),I2C_SCL_RSTART_SETUP_TIME_S)
		
#define I2C_GET_SCL_STOP_HOLD_TIME(i)  GET_PERI_REG_BITS2(I2C_SCL_STOP_HOLD_REG(i) , I2C_SCL_STOP_HOLD_TIME_V, I2C_SCL_STOP_HOLD_TIME_S)
#define I2C_SET_SCL_STOP_HOLD_TIME(i,val)  SET_PERI_REG_BITS(I2C_SCL_STOP_HOLD_REG(i) ,I2C_SCL_STOP_HOLD_TIME_V,(val),I2C_SCL_STOP_HOLD_TIME_S)
		
#define I2C_GET_SCL_STOP_SETUP_TIME(i)  GET_PERI_REG_BITS2(I2C_SCL_STOP_SETUP_REG(i) , I2C_SCL_STOP_SETUP_TIME_V, I2C_SCL_STOP_SETUP_TIME_S)
#define I2C_SET_SCL_STOP_SETUP_TIME(i,val)  SET_PERI_REG_BITS(I2C_SCL_STOP_SETUP_REG(i) ,I2C_SCL_STOP_SETUP_TIME_V,(val),I2C_SCL_STOP_SETUP_TIME_S)
		
#define I2C_GET_SCL_FILTER_EN(i)  GET_PERI_REG_BITS2(I2C_SCL_FILTER_CFG_REG(i) , I2C_SCL_FILTER_EN_V, I2C_SCL_FILTER_EN_S)
#define I2C_SET_SCL_FILTER_EN(i,val)  SET_PERI_REG_BITS(I2C_SCL_FILTER_CFG_REG(i) ,I2C_SCL_FILTER_EN_V,(val),I2C_SCL_FILTER_EN_S)
#define I2C_GET_SCL_FILTER_THRES(i)  GET_PERI_REG_BITS2(I2C_SCL_FILTER_CFG_REG(i) , I2C_SCL_FILTER_THRES_V, I2C_SCL_FILTER_THRES_S)
#define I2C_SET_SCL_FILTER_THRES(i,val)  SET_PERI_REG_BITS(I2C_SCL_FILTER_CFG_REG(i) ,I2C_SCL_FILTER_THRES_V,(val),I2C_SCL_FILTER_THRES_S)
		
#define I2C_GET_SDA_FILTER_EN(i)  GET_PERI_REG_BITS2(I2C_SDA_FILTER_CFG_REG(i) , I2C_SDA_FILTER_EN_V, I2C_SDA_FILTER_EN_S)
#define I2C_SET_SDA_FILTER_EN(i,val)  SET_PERI_REG_BITS(I2C_SDA_FILTER_CFG_REG(i) ,I2C_SDA_FILTER_EN_V,(val),I2C_SDA_FILTER_EN_S)
#define I2C_GET_SDA_FILTER_THRES(i)  GET_PERI_REG_BITS2(I2C_SDA_FILTER_CFG_REG(i) , I2C_SDA_FILTER_THRES_V, I2C_SDA_FILTER_THRES_S)
#define I2C_SET_SDA_FILTER_THRES(i,val)  SET_PERI_REG_BITS(I2C_SDA_FILTER_CFG_REG(i) ,I2C_SDA_FILTER_THRES_V,(val),I2C_SDA_FILTER_THRES_S)
		


void i2c_intr_config();
void i2c1_rx_intr_handler();
void I2C_Init(uint8_t i2c_no,I2C_InitTypeDef * I2C_InitStruct);
void I2C_ResetTxFifo(uint8_t i2c_no );
void I2C_TxFifoPush(uint8_t i2c_no,uint8_t *data , uint8_t len);
void I2C_ConfigCmd(uint8_t i2c_no,uint8_t cmd_idx, struct I2C_CmdDef* cmd);
void I2C_Start(uint8_t i2c_no);
void I2C_WaitTxDone(uint8_t i2c_no, uint8_t cmd_no);



#endif
